@@ -1,4 +1,4 @@
-/* Bug-buddy bug submitting program
+/* bug-buddy bug submitting program
  *
  * Copyright (C) 1999 - 2001 Jacob Berkman
  * Copyright 2000, 2001 Ximian, Inc.
@@ -122,6 +122,74 @@ buddy_error (GtkWidget *parent, const char *msg, ...)
 	g_free (s);
 }
 
+static void
+copy_review (GtkWidget *button, gpointer data)
+{
+	GtkTextView *text_view;
+	GtkTextBuffer *buffer;
+	GtkTextIter start;
+	GtkTextIter end;
+	GladeXML *xml = (GladeXML*) data;
+
+	text_view = GTK_TEXT_VIEW (glade_xml_get_widget (xml, "gdb-text"));
+	buffer = gtk_text_view_get_buffer (text_view);
+	gtk_text_buffer_get_start_iter (buffer, &start);
+	gtk_text_buffer_get_end_iter (buffer, &end);
+	gtk_text_buffer_select_range (buffer, &start, &end);
+	gtk_text_buffer_copy_clipboard (buffer, gtk_clipboard_get (GDK_NONE));
+	
+}
+
+static void
+edit_review (GtkWidget *button, gpointer data)
+{
+	GtkTextView *text_view;
+	GladeXML *xml = (GladeXML*) data;
+
+	text_view = GTK_TEXT_VIEW (glade_xml_get_widget (xml, "gdb-text"));
+	gtk_text_view_set_editable (text_view,
+				    gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (button)));
+
+
+}
+
+static void
+close_review (GtkWidget *button, gpointer data)
+{
+	GtkWidget *review_dialog = GTK_WIDGET (data);
+
+	gtk_widget_hide (review_dialog);
+}
+
+static void
+show_review (GtkWidget *button, gpointer data)
+{
+	GtkWidget *review_dialog, *main_window;
+	GtkWidget *edit, *copy, *close;
+	GladeXML *xml = (GladeXML*) data;
+	static gboolean initialized = FALSE;
+
+	if (!initialized) {
+		review_dialog = glade_xml_get_widget (xml, "review-dialog");
+		main_window = glade_xml_get_widget (xml, "main-window");
+		copy = glade_xml_get_widget (xml, "copy-review-button");
+		edit = glade_xml_get_widget (xml, "edit-review-button");
+		close = glade_xml_get_widget (xml, "close-review-button");
+
+		gtk_window_set_transient_for (GTK_WINDOW (review_dialog), GTK_WINDOW (main_window));
+
+		g_signal_connect (G_OBJECT (copy), "clicked", G_CALLBACK (copy_review), xml);
+		g_signal_connect (G_OBJECT (edit), "toggled", G_CALLBACK (edit_review), xml);
+		g_signal_connect (G_OBJECT (close), "clicked", G_CALLBACK (close_review), review_dialog);
+		
+		initialized = TRUE;
+	}
+
+	gtk_widget_show (glade_xml_get_widget (xml, "review-dialog"));
+
+}
+
+
 static GnomeVersionInfo*
 get_gnome_version_info (void)
 {
@@ -207,7 +275,7 @@ link_callback (GtkLinkButton *button, gpointer user_data)
 	if (gnome_vfs_url_show (link) != GNOME_VFS_OK) {
 		char *text;
 
-		text = g_markup_printf_escaped (_("Bug-buddy was unable to view the link \"%s\"\n"), link);
+		text = g_markup_printf_escaped (_("Bug Buddy was unable to view the link \"%s\"\n"), link);
 		buddy_error (NULL, text);
 		g_free (text);
 	}
@@ -422,10 +490,10 @@ bug_sent (SoupMessage *msg, GladeXML *xml)
 				                                  "renamed.  Please upgrade to the latest version."));
 				} else if (g_str_equal (ptr, "require_summary")) {
 					text = g_strdup_printf (_("The summary is required in your bug report. "
-				                                  "This should not happen with the latest bug buddy."));
+				                                  "This should not happen with the latest Bug Buddy."));
 				} else if (g_str_equal (ptr, "description_required")) {
 					text = g_strdup_printf (_("The description is required in your bug report. "
-				                                  "This should not happen with the latest bug buddy."));
+				                                  "This should not happen with the latest Bug Buddy."));
 				} else {
 					text = g_strdup_printf (_("The fault code returned by Bugzilla is not recognized. "
 				                                  "Please report the following information to "
@@ -576,15 +644,16 @@ send_report (BugzillaApplication *app, GnomeVersionInfo *gnome_version, GladeXML
 	gtk_text_buffer_get_end_iter (buffer, &end);
 	details_text = gtk_text_buffer_get_text (buffer, &start, &end, FALSE);
 
-	final_text = g_strdup_printf ("What were you doing when the application crashed?\n%s\n\n\n%s", 
+	type = GPOINTER_TO_INT (g_object_get_data (G_OBJECT(xml), "type"));
+	final_text = g_strdup_printf ("%s%s\n\n\n%s", 
+				      type == BUG_TYPE_CRASH ? "What were you doing when the application crashed?\n" : "",
 				      details_text != NULL ? details_text : "",
 				      gdb_text != NULL ? gdb_text : "<empty backtrace>");
 	
 	email = gtk_entry_get_text (GTK_ENTRY (glade_xml_get_widget (xml, "email-entry")));
-	type = GPOINTER_TO_INT (g_object_get_data (G_OBJECT(xml), "type"));
 	title = create_report_title (app, type, details_text);
 
-	message = bugzilla_create_report (app, gnome_version, email, title, final_text, xml, &err);
+	message = bugzilla_create_report (app, type, gnome_version, email, title, final_text, xml, &err);
 	if (message == NULL) {
 		char *text;
 
@@ -746,7 +815,7 @@ on_send_clicked (GtkWidget *button, gpointer data)
 
 	gtk_widget_show (glade_xml_get_widget (xml, "progressbar"));
 	gtk_widget_hide (glade_xml_get_widget (xml, "final-box"));
-	gtk_widget_hide (glade_xml_get_widget (xml, "expander1"));
+	gtk_widget_hide (glade_xml_get_widget (xml, "review-box"));
 
 	gtk_widget_size_request (glade_xml_get_widget (xml, "main-window"), &requisition);
 	gtk_window_resize (GTK_WINDOW (glade_xml_get_widget (xml, "main-window")),
@@ -809,10 +878,7 @@ known_app_finished (GladeXML *xml)
 				    "the application crashed.\n\n"
 
 				    "A valid email address is required.  This will allow developers to "
-				    "contact you for more information if necessary.\n\n"
-				    "Sensitive data may be present in the debug information, so please "
-				    "review details below if you are concerned about transmitting "
-				    "passwords or other sensitive data.\n"),
+				    "contact you for more information if necessary.\n\n"),
 				  strcmp (lang_note, en_lang_note) ? lang_note : NULL,
 				  NULL);
 
@@ -839,6 +905,9 @@ known_app_finished (GladeXML *xml)
 		gtk_widget_set_sensitive (button, FALSE);
 	}
 
+	g_signal_connect (glade_xml_get_widget (xml, "review-button"), "clicked", G_CALLBACK (show_review), xml);
+	gtk_widget_show (glade_xml_get_widget (xml, "review-box"));
+
 	gtk_widget_grab_focus (glade_xml_get_widget (xml, "details-view"));
 }
 
@@ -929,9 +998,6 @@ unknown_app_finished (GladeXML *xml)
 	g_signal_connect (GTK_BUTTON (button), "clicked", G_CALLBACK (on_save_clicked), xml);
 	gtk_widget_show (glade_xml_get_widget (xml, "send-button"));
 
-	/* expand the details about the stack trace */
-	gtk_expander_set_expanded (GTK_EXPANDER (glade_xml_get_widget (xml, "expander1")), TRUE);
-
 	gtk_statusbar_pop (GTK_STATUSBAR (glade_xml_get_widget (xml, "statusbar")), DEBUGGING);
 
 	gtk_label_set_text (GTK_LABEL (glade_xml_get_widget (xml, "main-text")),
@@ -984,6 +1050,38 @@ close_callback (GtkWidget *widget, gpointer user_data)
 	gtk_main_quit ();
 }
 
+static void
+help_callback (GtkWidget *widget, gpointer user_data)
+{
+	GnomeProgram *program = (GnomeProgram *)user_data;
+	GError *error = NULL;
+
+	gnome_help_display_desktop (program, 
+	                            "user-guide",
+	                            "user-guide", 
+	                            "feedback-bugs",
+	                            &error);
+
+	if (error) {
+		GtkWidget *error_dialog = 
+		    gtk_message_dialog_new (NULL, 
+		                            GTK_DIALOG_MODAL,
+		                            GTK_MESSAGE_ERROR, 
+		                            GTK_BUTTONS_CLOSE,
+		                            _("There was an error displaying help: %s"),
+		                            error->message);
+
+		g_signal_connect (G_OBJECT (error_dialog), "response", 
+		                  G_CALLBACK (gtk_widget_destroy), NULL);
+
+		gtk_window_set_resizable (GTK_WINDOW (error_dialog), FALSE);
+
+		gtk_widget_show (error_dialog);
+		g_error_free (error);
+		error = NULL;
+	}
+}
+
 static gboolean
 delete_callback (GtkWidget *widget, GdkEvent *event, gpointer data)
 {
@@ -1023,6 +1121,52 @@ fill_gnome_info (BugzillaApplication *app, GnomeVersionInfo *gnome_version, Glad
 	g_free (version_info);
 }
 
+static void
+fill_custom_info (BugzillaApplication *app, GladeXML *xml)
+{
+	GtkTextView   *text_view;
+	GtkTextIter    end;
+	GtkTextBuffer *buffer;
+	gint   status;
+	gchar *output;
+	gchar *standard_output = NULL;
+	gchar *standard_error = NULL;
+	GError *error = NULL;
+
+	g_return_if_fail (app != NULL);
+	g_return_if_fail (xml != NULL);
+
+	if (app->extra_info_script == NULL) {
+		return;
+	}
+
+	text_view = GTK_TEXT_VIEW (glade_xml_get_widget (xml, "gdb-text"));
+	buffer = gtk_text_view_get_buffer (text_view);
+	gtk_text_buffer_get_end_iter (buffer, &end);
+
+	if (!g_spawn_command_line_sync (app->extra_info_script, &standard_output, &standard_error,
+				        &status, &error)) {
+		gchar *error_string = g_strdup_printf ("There was an error running \"%s\" script:\n"
+                                                       "%s", app->extra_info_script, error->message);
+		gtk_text_buffer_insert (buffer, &end, error_string, strlen (error_string));
+
+		g_free (error);
+		g_free (error_string);
+		return;
+	}
+
+	output = g_strdup_printf ("Output of custom script \"%s\":\n"
+				  "%s\n\n",
+				  app->extra_info_script,
+				  standard_output ? standard_output : "");
+
+	gtk_text_buffer_insert (buffer, &end, output, strlen (output));
+
+	g_free (output);
+	g_free (standard_output);
+	g_free (standard_error);
+}
+
 
 static void
 fill_proccess_info (pid_t pid, GladeXML *xml)
@@ -1206,7 +1350,36 @@ fill_stderr_info (GladeXML *xml)
 }
 
 
+static GdkPixbuf*
+load_icon (const char *icon)
+{
+	GdkPixbuf *pixbuf = NULL;
 
+	if (g_path_is_absolute (icon)) {
+		pixbuf = gdk_pixbuf_new_from_file_at_size (icon, 48, 48, NULL);
+	}
+	if (pixbuf == NULL) {
+		pixbuf = gtk_icon_theme_load_icon (gtk_icon_theme_get_default (),
+						   icon, 48, 0, NULL);
+	}
+	if (pixbuf == NULL && strrchr (icon, '.') != NULL) {
+		char *name;
+		name = g_strndup (icon, strlen (icon) - strlen (strrchr (icon, '.')));
+		pixbuf = gtk_icon_theme_load_icon (gtk_icon_theme_get_default (),
+						   name, 48, 0, NULL);
+		g_free (name);
+	}
+
+	if (pixbuf == NULL) {
+		char *filename;
+		filename = g_strdup_printf (BUDDY_ICONDIR"/pixmaps/%s", icon);
+		pixbuf = gdk_pixbuf_new_from_file_at_size (filename, 48, 48, NULL);
+		g_free (filename);
+	}
+
+	return pixbuf;
+}
+	
 
 int
 main (int argc, char *argv[])
@@ -1244,7 +1417,7 @@ main (int argc, char *argv[])
 	                              GNOME_PARAM_APP_DATADIR, REAL_DATADIR,
 	                              NULL);
 
-	g_set_application_name (_("Bug-Buddy"));
+	g_set_application_name (_("Bug Buddy"));
 	gtk_window_set_default_icon_name ("bug-buddy");
 
 	s = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_APP_DATADIR,
@@ -1300,6 +1473,10 @@ main (int argc, char *argv[])
 	
 	g_object_set_data (G_OBJECT (xml), "gnome-version", gnome_version);
 
+	/* connect the signal handler for the help button */
+	g_signal_connect (glade_xml_get_widget (xml, "help-button"), "clicked", 
+	                  G_CALLBACK (help_callback), program);
+
 	gtk_widget_show (main_window);
 
 	apps = load_applications ();
@@ -1318,10 +1495,17 @@ main (int argc, char *argv[])
 		
 			g_object_set_data (G_OBJECT (xml), "app", app);
 	
-			if (app->pixbuf)
-				gtk_image_set_from_pixbuf (GTK_IMAGE (glade_xml_get_widget (xml, "app-image")),
-			                           		app->pixbuf);
+			if (app->icon) {
+				GdkPixbuf *pixbuf;
+				pixbuf = load_icon (app->icon);
+				if (pixbuf) {
+					gtk_image_set_from_pixbuf (GTK_IMAGE (glade_xml_get_widget (xml, "app-image")),
+			                           	   	   pixbuf);
+					g_object_unref(pixbuf);
+				}
+			}
 			fill_gnome_info (app, gnome_version, xml);
+			fill_custom_info (app, xml);
 		}
 
 		gtk_statusbar_push (GTK_STATUSBAR (statusbar),
@@ -1342,7 +1526,7 @@ main (int argc, char *argv[])
 			}
 
 			if (source_id == 0) {
-				buddy_error (NULL, _("Bug buddy encountered the following error when trying "
+				buddy_error (NULL, _("Bug Buddy encountered the following error when trying "
 		                     		     "to retrieve debugging information: %s\n"), err->message);
 				g_error_free (err);
 				g_object_unref (program);
@@ -1393,10 +1577,18 @@ main (int argc, char *argv[])
 		
 		g_object_set_data (G_OBJECT (xml), "app", app);
 	
-		if (app->pixbuf)
-			gtk_image_set_from_pixbuf (GTK_IMAGE (glade_xml_get_widget (xml, "app-image")),
-	                           		   app->pixbuf);
+		if (app->icon) {
+			GdkPixbuf *pixbuf;
+			
+			pixbuf = load_icon (app->icon);
+			if (pixbuf) {
+				gtk_image_set_from_pixbuf (GTK_IMAGE (glade_xml_get_widget (xml, "app-image")),
+							   pixbuf);
+				g_object_unref (pixbuf);
+			}
+		}
 		fill_gnome_info (app, gnome_version, xml);
+		fill_custom_info (app, xml);
 	
 		gtk_widget_show (glade_xml_get_widget (xml, "final-box"));
 		gtk_widget_show (glade_xml_get_widget (xml, "send-button"));
@@ -1411,7 +1603,8 @@ main (int argc, char *argv[])
 		gtk_label_set_text (GTK_LABEL (glade_xml_get_widget (xml, "main-text")), s);
 		g_free (s);
 
-		s = g_markup_printf_escaped (_("<span weight=\"bold\">Suggestion / Error description:</span>"));
+		s = g_markup_printf_escaped ("<span weight=\"bold\">%s</span>",
+                                             _("Suggestion / Error description:"));
 		gtk_label_set_markup (GTK_LABEL (glade_xml_get_widget (xml, "main-label")), s);
 		g_free (s);
 		
